@@ -398,10 +398,6 @@ setMethod("genericExpAddEntities", "MxExpectationStateSpace",
                 value <- unique(c(job@options[[key]], getAllModelNames(job))) # add every model in the whole tree to to no sort
                 # This is the nuclear option: whenever any model anywhere in the model tree has a SSMexp, don't sort any data.
                 job <- mxOption(job, key, value)
-                
-                # Run state space models single threaded
-                key <- "Number of Threads"
-                job <- mxOption(job, key, 1)
                 return(job)
         }
 )
