@@ -18,18 +18,25 @@ setClass(Class = "MxFitFunctionGREML",
            dV = "MxCharOrNumber",
            dVnames = "character",
            MLfit = "numeric",
-           numObs = "integer"),
+           numObs = "integer",
+           aug = "MxCharOrNumber",
+           augGrad = "MxCharOrNumber",
+           augHess = "MxCharOrNumber"),
          contains = "MxBaseFitFunction")
 
 
 setMethod("initialize", "MxFitFunctionGREML",
-          function(.Object, name = 'fitfunction', dV=character(0), MLfit=0, vector=FALSE, numObs=0L) {
+          function(.Object, name = 'fitfunction', dV=character(0), MLfit=0, vector=FALSE, numObs=0L, aug=character(0),
+          				 augGrad=character(0), augHess=character(0)) {
             .Object@name <- name
             .Object@dV <- dV
             .Object@dVnames <- as.character(names(dV))
             .Object@MLfit <- MLfit
             .Object@vector <- vector
             .Object@numObs <- numObs
+            .Object@aug <- aug
+            .Object@augGrad <- augGrad
+            .Object@augHess <- augHess
             return(.Object)
           }
 )
@@ -42,6 +49,13 @@ setMethod("qualifyNames", signature("MxFitFunctionGREML"),
               .Object@dV <- sapply(.Object@dV, imxConvertIdentifier, modelname, namespace)
               .Object@dVnames <- names(.Object@dV)
             }
+            if(length(.Object@aug)){.Object@aug <- imxConvertIdentifier(.Object@aug[1],modelname,namespace)}
+            if(length(.Object@augGrad)){
+            	.Object@augGrad <- imxConvertIdentifier(.Object@augGrad[1],modelname,namespace)
+            }
+            if(length(.Object@augHess)){
+            	.Object@augHess <- imxConvertIdentifier(.Object@augHess[1],modelname,namespace)
+            }
             return(.Object)
           })
 
@@ -50,6 +64,13 @@ setMethod("genericFitRename", signature("MxFitFunctionGREML"),
             if(length(.Object@dV)){
               .Object@dV <- sapply(.Object@dV, renameReference, oldname, newname)
             }
+          	if(length(.Object@aug)){.Object@aug <- renameReference(.Object@aug[1], oldname, newname)}
+          	if(length(.Object@augGrad)){
+          		.Object@augGrad <- renameReference(.Object@augGrad[1], oldname, newname)
+          	}
+          	if(length(.Object@augHess)){
+          		.Object@augHess <- renameReference(.Object@augHess[1], oldname, newname)
+          	}
             return(.Object)
           })
 
@@ -81,6 +102,13 @@ setMethod("genericFitFunConvert", "MxFitFunctionGREML",
             if(length(.Object@dV)){
               .Object@dV <- sapply(.Object@dV, imxLocateIndex, model=flatModel, referant=name)
             }
+            if(length(.Object@aug)){.Object@aug <- imxLocateIndex(.Object@aug[1], model=flatModel, referant=name)}
+            if(length(.Object@augGrad)){
+            	.Object@augGrad <- imxLocateIndex(.Object@augGrad[1], model=flatModel, referant=name)
+            }
+            if(length(.Object@augHess)){
+            	.Object@augHess <- imxLocateIndex(.Object@augHess[1], model=flatModel, referant=name)
+            }
             return(.Object)
           })
 
@@ -94,6 +122,6 @@ setMethod("generateReferenceModels", "MxFitFunctionGREML",
 					})
 
 
-mxFitFunctionGREML <- function(dV=character(0)){
-  return(new("MxFitFunctionGREML",dV=dV))
+mxFitFunctionGREML <- function(dV=character(0), aug=character(0), augGrad=character(0), augHess=character(0)){
+  return(new("MxFitFunctionGREML",dV=dV,aug=aug,augGrad=augGrad,augHess=augHess))
 }
