@@ -35,22 +35,26 @@ mxRestore <- function(model, chkpt.directory = ".", chkpt.prefix = "", line=NULL
 		stop("'model' argument must be a MxModel object")
 	}
 	chkpt.directory <- removeTrailingSeparator(chkpt.directory)
+	if (strict && chkpt.prefix == "") {
+		chkpt.prefix <- model$name
+	}
 	pattern <- paste("^\\Q", chkpt.prefix, "\\E.*(\\.omx)$", sep = '')
 	chkpt.files <- list.files(chkpt.directory, full.names = FALSE)
 	chkpt.files <- grep(pattern, chkpt.files, perl=TRUE, value=TRUE)
-	if(length(chkpt.files) == 0) {
-		return(model)
-	}
-	matchIndex <- match(paste(model$name, 'omx', sep="."), chkpt.files)
 	if (strict) {
-		if (!is.na(matchIndex)) {
-			chkpt.files <- chkpt.files[matchIndex]
-		} else {
+		if (length(chkpt.files) > 1) {
+			stop(paste("chkpt.prefix", omxQuotes(chkpt.prefix),
+				   "matched more than one file"))
+		} else if (length(chkpt.files) == 0) {
 			stop(paste("Cannot find", omxQuotes(paste(model$name, 'omx', sep=".")),
 				   "in", chkpt.directory))
 		}
 	} else {
+		if(length(chkpt.files) == 0) {
+			return(model)
+		}
 		# Move the most likely match to the end so those estimates take precedence.
+		matchIndex <- match(paste(model$name, 'omx', sep="."), chkpt.files)
 		chkpt.files <- c(chkpt.files[-matchIndex], paste(model$name, 'omx', sep="."))
 	}
 	if (length(chkpt.files) > 1 && !is.null(line)) {
