@@ -669,6 +669,8 @@ void FitContext::init()
 	CI = NULL;
 	targetFit = nan("uninit");
 	lowerBound = false;
+	openmpUser = false;
+	computeCount = 0;
 
 	hess.resize(numParam, numParam);
 	ihess.resize(numParam, numParam);
@@ -700,7 +702,6 @@ void FitContext::allocStderrs()
 {
 	if (stderrs) return;
 
-	size_t numParam = varGroup->vars.size();
 	stderrs = new double[numParam];
 
 	for (size_t px=0; px < numParam; ++px) {
@@ -713,9 +714,9 @@ FitContext::FitContext(omxState *_state, std::vector<double> &startingValues)
 	parent = NULL;
 	varGroup = Global->findVarGroup(FREEVARGROUP_ALL);
 	init();
+	profiledOut.assign(numParam, false);
 
 	state = _state;
-	size_t numParam = varGroup->vars.size();
 	if (startingValues.size() != numParam) {
 		Rf_error("Got %d starting values for %d parameters",
 		      startingValues.size(), numParam);
@@ -735,22 +736,28 @@ FitContext::FitContext(FitContext *parent, FreeVarGroup *varGroup)
 	size_t dvars = varGroup->vars.size();
 	if (dvars == 0) return;
 	mapToParent.resize(dvars);
+	profiledOut.resize(numParam);
 
 	size_t d1 = 0;
 	for (size_t s1=0; s1 < src->vars.size(); ++s1) {
 		if (src->vars[s1] != dest->vars[d1]) continue;
 		mapToParent[d1] = s1;
 		est[d1] = parent->est[s1];
+		profiledOut[d1] = parent->profiledOut[s1];
 		if (++d1 == dvars) break;
 	}
 	if (d1 != dvars) Rf_error("Parent free parameter group (id=%d) is not a superset of %d",
 			       src->id[0], dest->id[0]);
 
+
 	wanted = parent->wanted;
 	infoDefinite = parent->infoDefinite;
 	infoCondNum = parent->infoCondNum;
 	iterations = parent->iterations;
+
+	// confidence interval stuff
 	CI = parent->CI;
+	compositeCIFunction = parent->compositeCIFunction;
 	targetFit = parent->targetFit;
 	lowerBound = parent->lowerBound;
 }
@@ -768,6 +775,8 @@ void FitContext::updateParent()
 	parent->infoDefinite = infoDefinite;
 	parent->infoCondNum = infoCondNum;
 	parent->iterations = iterations;
+	parent->computeCount += computeCount;
+	computeCount = 0;
 
 	// rewrite using mapToParent TODO
 
@@ -790,6 +799,19 @@ void FitContext::updateParent()
 	// pda(parent->est, 1, dvars);
 }
 
+int FitContext::getComputeCount()
+{
+	if (isClone()) {
+		return parent->getComputeCount();
+	} else {
+		int cc = computeCount;
+		for (size_t cx=0; cx < childList.size(); ++cx) {
+			cc += childList[cx]->computeCount;
+		}
+		return cc;
+	}
+}
+
 void FitContext::updateParentAndFree()
 {
 	updateParent();
@@ -943,7 +965,7 @@ void copyParamToModelInternal(FreeVarGroup *varGroup, omxState *os, double *at)
 		std::string buf;
 		buf += string_snprintf("copyParamToModel: c(");
 		for(size_t k = 0; k < numParam; k++) {
-			buf += string_snprintf("%f", at[k]);
+			buf += string_snprintf("%g", at[k]);
 			if (k < numParam-1) buf += string_snprintf(", ");
 		}
 		buf += (")\n");
@@ -966,7 +988,7 @@ void FitContext::copyParamToModelClean()
 
 	if (RFitFunction) omxRepopulateRFitFunction(RFitFunction, est, numParam);
 
-	if (childList.size() == 0) return;
+	if (childList.size() == 0 || !openmpUser) return;
 
 	for(size_t i = 0; i < childList.size(); i++) {
 		memcpy(childList[i]->est, est, sizeof(double) * numParam);
@@ -1065,21 +1087,43 @@ void FitContext::postInfo()
 	}
 }
 
+bool FitContext::isClone() const
+{
+	return state->isClone();
+}
+
 void FitContext::createChildren()
 {
-	if (Global->numThreads <= 1) return;
+	if (Global->numThreads <= 1) {
+		diagParallel(OMX_DEBUG, "FitContext::createChildren: max threads set to 1");
+		return;
+	}
+	if (childList.size()) return;
 
 	for(size_t j = 0; j < state->expectationList.size(); j++) {
-		if (!state->expectationList[j]->canDuplicate) return;
+		if (!state->expectationList[j]->canDuplicate) {
+			diagParallel(OMX_DEBUG, "FitContext::createChildren: %s cannot be duplicated",
+				     state->expectationList[j]->name);
+			return;
+		}
 	}
 	for(size_t j = 0; j < state->algebraList.size(); j++) {
 		omxFitFunction *ff = state->algebraList[j]->fitFunction;
-		if (ff && !ff->canDuplicate) return;
+		if (!ff) continue;
+		if (!ff->canDuplicate) {
+			diagParallel(OMX_DEBUG, "FitContext::createChildren: %s cannot be duplicated",
+				     state->algebraList[j]->name());
+			return;
+		}
+		if (ff->openmpUser) {
+			diagParallel(OMX_DEBUG, "FitContext::createChildren: %s is an OpenMP user",
+				     state->algebraList[j]->name());
+		}
+		openmpUser |= ff->openmpUser;
 	}
 
-	if (childList.size()) return;
-
-	if (OMX_DEBUG) mxLog("Create %d FitContext for parallel processing", Global->numThreads);
+	diagParallel(OMX_DEBUG, "FitContext::createChildren: create %d FitContext for parallel processing; OpenMP user=%d",
+		     Global->numThreads, openmpUser);
 
 	int numThreads = Global->numThreads;
 
@@ -1094,7 +1138,7 @@ void FitContext::createChildren()
 		//if (OMX_DEBUG) mxLog("Protect depth at line %d: %d", __LINE__, mpi.getDepth());
 	}
 
-	if (OMX_DEBUG) mxLog("Done creating %d omxState", Global->numThreads);
+	if (OMX_DEBUG) mxLog("FitContext::createChildren: done creating %d omxState", Global->numThreads);
 }
 
 void FitContext::destroyChildren()
@@ -2930,15 +2974,10 @@ void GradientOptimizerContext::reset()
 GradientOptimizerContext::GradientOptimizerContext(FitContext *fc, int verbose)
 	: fc(fc), verbose(verbose)
 {
-	if (fc->profiledOut.size() == 0) {
-		fc->profiledOut.assign(fc->numParam, false);
-		numFree = fc->numParam;
-	} else {
-		numFree = 0;
-		for (size_t vx=0; vx < fc->profiledOut.size(); ++vx) {
-			if (fc->profiledOut[vx]) continue;
-			++numFree;
-		}
+	numFree = 0;
+	for (size_t vx=0; vx < fc->profiledOut.size(); ++vx) {
+		if (fc->profiledOut[vx]) continue;
+		++numFree;
 	}
 	optName = "?";
 	fitMatrix = NULL;
@@ -2954,6 +2993,7 @@ GradientOptimizerContext::GradientOptimizerContext(FitContext *fc, int verbose)
 	est.resize(numFree);
 	grad.resize(numFree);
 	copyToOptimizer(est.data());
+	numOptimizerThreads = (fc->childList.size() && !fc->openmpUser)? fc->childList.size() : 1;
 	reset();
 }
 
@@ -2995,15 +3035,15 @@ void GradientOptimizerContext::useBestFit()
 	// restore gradient too? TODO
 }
 
-void GradientOptimizerContext::copyFromOptimizer(double *myPars)
+void GradientOptimizerContext::copyFromOptimizer(double *myPars, FitContext *fc2)
 {
 	int px=0;
-	for (size_t vx=0; vx < fc->profiledOut.size(); ++vx) {
-		if (fc->profiledOut[vx]) continue;
-		fc->est[vx] = myPars[px];
+	for (size_t vx=0; vx < fc2->profiledOut.size(); ++vx) {
+		if (fc2->profiledOut[vx]) continue;
+		fc2->est[vx] = myPars[px];
 		++px;
 	}
-	fc->copyParamToModel();
+	fc2->copyParamToModel();
 }
 
 void GradientOptimizerContext::finish()
@@ -3021,6 +3061,22 @@ void GradientOptimizerContext::finish()
 	fc->copyParamToModel();
 }
 
+double GradientOptimizerContext::evalFit(double *myPars, int thrId, int *mode)
+{
+	FitContext *fc2 = thrId >= 0? fc->childList[thrId] : fc;
+	Eigen::Map< Eigen::VectorXd > Est(myPars, fc2->numParam);
+	copyFromOptimizer(myPars, fc2);
+	int want = FF_COMPUTE_FIT;
+	ComputeFit(optName, fc2->lookupDuplicate(fitMatrix), want, fc2);
+	if (fc2->outsideFeasibleSet() || isErrorRaised()) {
+		*mode = -1;
+	}
+	if (verbose >= 3) {
+		mxLog("fit %f (mode %d)", fc2->fit, *mode);
+	}
+	return fc2->fit;
+}
+
 double GradientOptimizerContext::solFun(double *myPars, int* mode)
 {
 	Eigen::Map< Eigen::VectorXd > Est(myPars, fc->numParam);
@@ -3031,7 +3087,7 @@ double GradientOptimizerContext::solFun(double *myPars, int* mode)
 	}
 
 	if (*mode == 1) fc->iterations += 1;
-	copyFromOptimizer(myPars);
+	copyFromOptimizer(myPars, fc);
 
 	int want = FF_COMPUTE_FIT;
 	// eventually want to permit analytic gradient during CI
