@@ -13,6 +13,181 @@
 //#include <Eigen/UmfPackSupport>
 //#include <RcppEigenCholmod.h>
 
+template <typename T1>
+class AsymTool {
+	int AshallowDepth;
+	double signA;
+	std::vector<T1> &latentFilter;
+	Eigen::SparseMatrix<double>      ident;
+	bool analyzed;
+	bool hasDeterminedDepth;
+	Eigen::SparseLU< Eigen::SparseMatrix<double>,
+		Eigen::COLAMDOrdering<Eigen::SparseMatrix<double>::Index> > Asolver;
+	//Eigen::UmfPackLU< Eigen::SparseMatrix<double> > Asolver;
+	int clumpObs;
+	bool filtered;
+	int invertCount;
+	int filterCount;
+ public:
+	bool determinedDepth() const { return hasDeterminedDepth; };
+	bool isFiltered() const { return filtered; };
+	int getDepth() const { return AshallowDepth; };
+	void setDepth(int depth) {
+		AshallowDepth = depth;
+		hasDeterminedDepth = true;
+		if (depth >= 0) signA = 1.0;
+	};
+	double getSign() const { return signA; };
+	int getInvertCount() const { return invertCount; };
+	int getFilterCount() const { return filterCount; };
+
+	Eigen::SparseMatrix<double> fullA;
+	Eigen::SparseMatrix<double> IAF;
+
+	AsymTool(std::vector<T1> &latentFilter) :
+	AshallowDepth(-1), signA(-1.0), latentFilter(latentFilter), analyzed(false), hasDeterminedDepth(false),
+		invertCount(0), filterCount(0) {};
+	void resize(int clumpVars, int _clumpObs)
+	{
+		fullA.resize(clumpVars, clumpVars);
+		ident.resize(clumpVars, clumpVars);
+		ident.setIdentity();
+		clumpObs = _clumpObs;
+	};
+	void determineShallowDepth(FitContext *fc);
+	void invert();
+	void filter();
+};
+
+template <typename T1> void AsymTool<T1>::determineShallowDepth(FitContext *fc)
+{
+	int maxDepth = std::min(fullA.cols(), 30);
+	if (Global->RAMMaxDepth != NA_INTEGER) maxDepth = Global->RAMMaxDepth;
+	Eigen::SparseMatrix<double> curProd = fullA;
+	for (int tx=1; tx <= maxDepth; ++tx) {
+		if (false) {
+			Eigen::MatrixXd tmp = curProd;
+			mxPrintMat("curProd", tmp);
+		}
+		curProd = (curProd * fullA.transpose()).eval();
+		bool allZero = true;
+		for (int k=0; k < curProd.outerSize(); ++k) {
+			for (Eigen::SparseMatrix<double>::InnerIterator it(curProd, k); it; ++it) {
+				if (it.value() != 0.0) {
+					allZero = false;
+					break;
+				}
+			}
+		}
+		if (allZero) {
+			AshallowDepth = tx - 1;
+			break;
+		}
+	}
+	fullA.setZero();
+
+	if (AshallowDepth >= 0) signA = 1.0;
+	hasDeterminedDepth = true;
+}
+
+template <typename T1> void AsymTool<T1>::invert()
+{
+	// consider http://users.clas.ufl.edu/hager/papers/Lightning/update.pdf ?
+	if (AshallowDepth >= 0) {
+		fullA.makeCompressed();
+		IAF = fullA + ident;
+		for (int iter=1; iter <= AshallowDepth; ++iter) {
+			IAF = (IAF * fullA + ident).eval();
+			//{ Eigen::MatrixXd tmp = out; mxPrintMat("out", tmp); }
+		}
+	} else {
+		fullA += ident;
+		if (!analyzed) {
+			analyzed = true;
+			fullA.makeCompressed();
+			Asolver.analyzePattern(fullA);
+		}
+		Asolver.factorize(fullA);
+		if (Asolver.info() != Eigen::Success) {
+			Rf_error("Failed to invert flattened A matrix; %s",
+				 Asolver.lastErrorMessage().c_str());
+		}
+
+		IAF = Asolver.solve(ident);
+		fullA -= ident;  // leave unchanged
+		//{ Eigen::MatrixXd tmp = out; mxPrintMat("out", tmp); }
+	}
+	filtered = false;
+	++invertCount;
+}
+
+template <typename T1> void AsymTool<T1>::filter()
+{
+	const bool doubleCheck = false;
+	Eigen::MatrixXd denseA;
+	if (doubleCheck) {
+		denseA = IAF;
+	}
+
+	// We built A transposed so we can quickly filter columns
+	// Switch to filterOuter http://eigen.tuxfamily.org/bz/show_bug.cgi?id=1130 TODO
+	IAF.uncompress();
+	Eigen::SparseMatrix<double>::Index *op = IAF.outerIndexPtr();
+	Eigen::SparseMatrix<double>::Index *nzp = IAF.innerNonZeroPtr();
+	int dx = 0;
+	for (int cx=0; cx < fullA.cols(); ++cx) {
+		if (!latentFilter[cx]) continue;
+		op[dx] = op[cx];
+		nzp[dx] = nzp[cx];
+		++dx;
+	}
+	op[dx] = op[fullA.cols()];
+	IAF.conservativeResize(fullA.rows(), clumpObs);
+
+	// I've screwed this up 3-4 times, better check it!
+	if (OMX_DEBUG && dx != clumpObs) Rf_error("latentFilter has wrong count %d != %d",
+						  dx, clumpObs);
+
+	if (doubleCheck) {
+		Eigen::MatrixXd denseAF;
+		denseAF.resize(fullA.rows(), clumpObs);
+		int dx=0;
+		for (int cx=0; cx < fullA.cols(); ++cx) {
+			if (!latentFilter[cx]) continue;
+			denseAF.col(dx) = denseA.col(cx);
+			++dx;
+		}
+		if (dx != clumpObs) Rf_error("latentFilter has wrong count %d != %d",
+					     dx, clumpObs);
+
+		// ensure inner iterator works
+		for (int k=0; k< IAF.outerSize(); ++k) {
+			for (Eigen::SparseMatrix<double>::InnerIterator it(IAF, k); it; ++it) {
+				if (denseAF.coeff(it.row(), it.col()) != it.value()) {
+					mxLog("[%d,%d] %f != %f",
+					      it.row(), it.col(), denseAF.coeff(it.row(), it.col()), it.value());
+				}
+			}
+		}
+
+		Eigen::MatrixXd denseFilteredA = IAF;
+		if ((denseAF.array() != denseFilteredA.array()).any()) {
+			for (int rx=0; rx<denseAF.rows(); ++rx) {
+				for (int cx=0; cx<denseAF.cols(); ++cx) {
+					if (denseAF.coeff(rx,cx) != denseFilteredA.coeff(rx,cx)) {
+						mxLog("[%d,%d] %f != %f",
+						      rx, cx, denseAF.coeff(rx,cx), denseFilteredA.coeff(rx,cx));
+					}
+				}
+			}
+			Rf_error("stop");
+		}
+	}
+	filtered = true;
+	++filterCount;
+	//{ Eigen::MatrixXd tmp = out; mxPrintMat("out", tmp); }
+}
+
 template<typename _MatrixType, int _UpLo = Eigen::Lower>
 class SimpCholesky : public Eigen::LDLT<_MatrixType, _UpLo> {
  private:
@@ -190,6 +365,8 @@ class SimpCholesky : public Eigen::SimplicialLDLT<_MatrixType, _UpLo> {
 	};
 */
 
+class omxRAMExpectation;
+
 namespace RelationalRAMExpectation {
 
 	// addrSetup and addr are conceptual the same object. They are
@@ -202,18 +379,18 @@ namespace RelationalRAMExpectation {
 		int numJoins;
 		int parent1;  // first parent
 		int fk1;      // first foreign key
+		bool rotationLeader;
 
 		// clump indexes into the layout for models that
 		// are considered a compound component of this model.
 		std::vector<int> clump;
 		bool clumped;
-		int region;
-		int group;
-		int copy;
 	};
 
-	struct addr {
-		omxExpectation *model;
+	class addr {
+	private:
+		omxExpectation *model;  // read-only
+	public:
 		int row;                 // to load definition variables (never the key)
 		struct independentGroup *ig;
 		int igIndex;
@@ -223,11 +400,28 @@ namespace RelationalRAMExpectation {
 		int numObs() const { return numObsCache; }
 		double rampartScale;
 
-		std::string modelName() const {
+		std::string modelName() {
 			std::string tmp = model->data->name;
 			tmp = tmp.substr(0, tmp.size() - 5); // remove ".data" suffix
 			return tmp;
 		};
+		void setModel(omxExpectation *ex) { model=ex; };
+		omxExpectation *getModel(FitContext *fc) {
+			return omxExpectationFromIndex(model->expNum, fc->state);
+		};
+		int getExpNum() const { return model->expNum; };
+		omxData *getData() const { return model->data; };
+		std::vector<bool> &getIgnoreDefVar();
+		omxRAMExpectation *getRAMExpectation(FitContext *fc) {
+			return (omxRAMExpectation*) getModel(fc)->argStruct;
+		};
+		omxRAMExpectation *getRAMExpectationReadOnly() const {
+			// NOTE: not per-thread!
+			return (omxRAMExpectation*) model->argStruct;
+		};
+		std::vector< omxMatrix* > &getBetween() const;
+		omxMatrix *getDataColumns() const { return model->dataColumns; };
+		void dataRow(omxMatrix *out) const;
 	};
 
 	struct RowToLayoutMapCompare {
@@ -240,28 +434,33 @@ namespace RelationalRAMExpectation {
 	};
 
 	struct placement {
-		int aIndex;      // index into addr vector
 		int modelStart;  // both latent and obs
 		int obsStart;
 	};
 
+	struct sufficientSet {
+		int                              start;   // index into placements
+		int                              length;  // # of clumpSize units
+		Eigen::MatrixXd                  dataCov;
+		Eigen::VectorXd                  dataMean;
+	};
+
 	class independentGroup {
 	private:
 		class state &st;
-		bool analyzed;
-		int AshallowDepth;
-		double signA;
-		Eigen::SparseMatrix<double>      ident;
 
 		void refreshModel(FitContext *fc);
 		void refreshUnitA(FitContext *fc, int px);
 		void invertAndFilterA();
 	public:
+		int arrayIndex;
 		typedef std::map< std::pair<omxData*,int>, int, RowToLayoutMapCompare> RowToPlacementMapType;
 		RowToPlacementMapType            rowToPlacementMap;
+		std::vector<int>                 gMap;  // vector of indexes into layout
 		std::vector<placement>           placements;
 		const int                        clumpSize;
 		int                              clumpVars, clumpObs;
+		std::vector<sufficientSet>       sufficientSets;
 		SEXP                             obsNameVec;
 		SEXP                             varNameVec;
 		// make dataColumn optional TODO
@@ -269,57 +468,82 @@ namespace RelationalRAMExpectation {
 		Eigen::VectorXd                  dataVec;
 		Eigen::VectorXd                  fullMean;
 		Eigen::VectorXd                  rawFullMean;
-		Eigen::VectorXd                  expectedMean;
+		Eigen::VectorXd                  expectedVec;
 		Eigen::SparseMatrix<double>      fullCov;
 		bool                             analyzedCov;
 		//Cholmod< Eigen::SparseMatrix<double> > covDecomp;
 		SimpCholesky< Eigen::MatrixXd >  covDecomp;
 		Eigen::SparseMatrix<double>      fullS;
-		std::vector<bool>                latentFilter; // use to reduce the A matrix
+		std::vector<bool>                latentFilter; // false when latent or missing
 
 		// could store coeff extraction plan in addr TODO
-		Eigen::SparseMatrix<double>      fullA;
-		Eigen::SparseLU< Eigen::SparseMatrix<double>,
-				 Eigen::COLAMDOrdering<Eigen::SparseMatrix<double>::Index> > Asolver;
-		Eigen::SparseMatrix<double>      IAF;
-		//Eigen::UmfPackLU< Eigen::SparseMatrix<double> > Asolver;
+		AsymTool<bool>          asymT;
 
 		independentGroup(class state *st, int size, int clumpSize)
-			: st(*st), analyzed(false), AshallowDepth(-1), signA(-1), clumpSize(clumpSize),
-			analyzedCov(false)
+			: st(*st), clumpSize(clumpSize),
+			analyzedCov(false), asymT(latentFilter)
 		{ placements.reserve(size); };
-		void prep(int maxSize, int totalObserved, FitContext *fc);
+		independentGroup(independentGroup *ig);
+		int numLooseClumps() {
+			independentGroup &par = getParent();
+			int loose = par.placements.size() / clumpSize;
+			if (par.sufficientSets.size()) {
+				loose = par.sufficientSets[0].start / clumpSize;
+			}
+			return loose;
+		};
+		void place(int ax);
+		void prep(FitContext *fc);
 		void determineShallowDepth(FitContext *fc);
 		int verbose() const;
 		void filterFullMean();
+		void finalizeData();
 		Eigen::SparseMatrix<double> getInputMatrix() const;
 		void computeCov1(FitContext *fc);
 		void computeCov2();
 		void exportInternalState(MxRList &out, MxRList &dbg);
+		independentGroup &getParent();
 	};
 
 	class state {
 	private:
+		state *parent;
 		std::vector<int>                 rampartUsage;
 		std::vector< std::vector<int> >  rotationPlan;
-		int                              totalObserved;
 
 	public:
+		typedef std::vector< std::set<int> > ConnectedType;
 		struct omxExpectation *homeEx;
+		std::set<struct omxExpectation *> allEx;
 		typedef std::map< std::pair<omxData*,int>, int, RowToLayoutMapCompare> RowToLayoutMapType;
 		RowToLayoutMapType               rowToLayoutMap;
 		std::vector<addrSetup>		 layoutSetup;
 		std::vector<addr>		 layout;
+
+		void clumpWith(int upper, int lower) {
+			if (layoutSetup[lower].clumped) Rf_error("%d is already clumped", lower);
+			layoutSetup[upper].clump.push_back(lower);
+			layoutSetup[lower].clumped = true;
+		};
+
 		omxMatrix                       *smallCol;
 		std::vector<independentGroup*>   group;
+		bool                             doIdentifyZeroVarPred;
 
 	private:
 		int flattenOneRow(omxExpectation *expectation, int frow, int &maxSize);
+		template <typename T>
+		void placeSet(std::set<std::vector<T> > &toPlace, independentGroup *ig);
 		void planModelEval(int maxSize, FitContext *fc);
+		void identifyZeroVarPred(FitContext *fc);
 		int rampartRotate(int level);
 		template <typename T> void oertzenRotate(std::vector<T> &t1);
 		template <typename T> void applyRotationPlan(T accessor);
 		template <typename T> void appendClump(int ax, std::vector<T> &clump);
+		template <typename T> void propagateDefVar(omxRAMExpectation *ram,
+							   Eigen::MatrixBase<T> &transition,
+							   omxRAMExpectation *ram2, bool within);
+		void computeConnected(std::vector<int> &region, ConnectedType &connected);
 	public:
 		~state();
 		void computeCov(FitContext *fc);
@@ -328,16 +552,26 @@ namespace RelationalRAMExpectation {
 		int verbose() const;
 		bool hasRotationPlan() const { return rotationPlan.size() != 0; }
 		void exportInternalState(MxRList &dbg);
+		state &getParent() { return *parent; };
 	};
 };
 
+typedef std::set< std::pair< omxExpectation*, int> > dvScoreboardSetType;
+
 class omxRAMExpectation {
 	bool trivialF;
 	int Zversion;
 	omxMatrix *_Z;
  public:
+	std::vector< dvScoreboardSetType > dvScoreboard;
+	Eigen::VectorXd hasVariance;
+	std::vector<bool> ignoreDefVar;
+	std::vector<bool> latentFilter; // false when latent
 
- omxRAMExpectation(omxMatrix *Z) : trivialF(false), Zversion(0), _Z(Z) {};
+	// composition of F permutation and expectation->dataColumns
+	Eigen::VectorXi dataCols;
+
+ 	omxRAMExpectation(omxMatrix *Z) : trivialF(false), Zversion(0), _Z(Z) {};
 	~omxRAMExpectation() {
 		omxFreeMatrix(_Z);
 	};
@@ -362,7 +596,7 @@ class omxRAMExpectation {
 	RelationalRAMExpectation::state *rram;
 	bool forceSingleGroup;
 
-	void ensureTrivialF();
+	void studyF(omxMatrix *dc);
 };
 
 namespace RelationalRAMExpectation {
@@ -372,6 +606,7 @@ namespace RelationalRAMExpectation {
 	}
 
 	inline int independentGroup::verbose() const { return st.verbose(); };
+
 };
 
 #endif
