@@ -4,14 +4,14 @@
 // See http://en.wikipedia.org/wiki/Finite_difference
 
 struct forward_difference_grad {
-	template <typename T1, typename T2>
-	void operator()(T1 ff, double refFit, Eigen::MatrixBase<T2> &point,
-			double offset, int px, int numIter, std::vector<double> &Gaprox)
+	template <typename T1>
+	void operator()(T1 ff, double refFit, int thrId, double *point,
+			double offset, int px, int numIter, double *Gaprox)
 	{
 		double orig = point[px];
 		for(int k = 0; k < numIter; k++) {
 			point[px] = orig + offset;
-			Gaprox[k] = (ff(point) - refFit) / offset;
+			Gaprox[k] = (ff(point, thrId) - refFit) / offset;
 			offset *= .5;
 		}
 		point[px] = orig;
@@ -19,16 +19,16 @@ struct forward_difference_grad {
 };
 
 struct central_difference_grad {
-	template <typename T1, typename T2>
-	void operator()(T1 ff, double refFit, Eigen::MatrixBase<T2> &point,
-			double offset, int px, int numIter, std::vector<double> &Gaprox)
+	template <typename T1>
+	void operator()(T1 ff, double refFit, int thrId, double *point,
+			double offset, int px, int numIter, double *Gaprox)
 	{
 		double orig = point[px];
 		for(int k = 0; k < numIter; k++) {
 			point[px] = orig + offset;
-			double f1 = ff(point);
+			double f1 = ff(point, thrId);
 			point[px] = orig - offset;
-			double f2 = ff(point);
+			double f2 = ff(point, thrId);
 			Gaprox[k] = (f1 - f2) / (2.0 * offset);
 			offset *= .5;
 		}
@@ -37,35 +37,42 @@ struct central_difference_grad {
 };
 
 template <typename T1, typename T2, typename T3, typename T4>
-void gradientImpl(T1 ff, double refFit, Eigen::MatrixBase<T2> &point, int numIter,
+void gradientImpl(T1 ff, int numThreads, double refFit, Eigen::MatrixBase<T2> &point, int numIter,
 		  const double eps, T4 dfn, Eigen::MatrixBase<T3> &gradOut)
 {
+	Eigen::MatrixXd grid(numIter, point.size());
+	numThreads = std::min(numThreads, point.size()); // could break work into smaller pieces TODO
+	Eigen::MatrixXd thrPoint(point.size(), numThreads);
+	thrPoint.colwise() = point;
+
+#pragma omp parallel for num_threads(numThreads)
 	for (int px=0; px < int(point.size()); ++px) {
+		int thrId = omp_get_thread_num();
+		int thrSelect = numThreads==1? -1 : thrId;
 		double offset = std::max(fabs(point[px] * eps), eps);
-		std::vector<double> Gaprox(numIter);
-		dfn(ff, refFit, point, offset, px, numIter, Gaprox);
+		dfn(ff, refFit, thrSelect, &thrPoint.coeffRef(0, thrId), offset, px, numIter, &grid.coeffRef(0,px));
 		for(int m = 1; m < numIter; m++) {						// Richardson Step
 			for(int k = 0; k < (numIter - m); k++) {
 				// NumDeriv Hard-wires 4s for r here. Why?
-				Gaprox[k] = (Gaprox[k+1] * pow(4.0, m) - Gaprox[k])/(pow(4.0, m)-1);
+				grid(k,px) = (grid(k+1,px) * pow(4.0, m) - grid(k,px))/(pow(4.0, m)-1);
 			}
 		}
-		gradOut[px] = Gaprox[0];
+		gradOut[px] = grid(0,px);
 	}
 }
 
 template <typename T1, typename T2, typename T3>
-void gradient_with_ref(GradientAlgorithm algo, int order, double eps, T1 ff, double refFit,
+void gradient_with_ref(GradientAlgorithm algo, int numThreads, int order, double eps, T1 ff, double refFit,
 		       Eigen::MatrixBase<T2> &point, Eigen::MatrixBase<T3> &gradOut)
 {
 	switch (algo) {
 	case GradientAlgorithm_Forward:{
 		forward_difference_grad dfn;
-		gradientImpl(ff, refFit, point, order, eps, dfn, gradOut);
+		gradientImpl(ff, numThreads, refFit, point, order, eps, dfn, gradOut);
 		break;}
 	case GradientAlgorithm_Central:{
 		central_difference_grad dfn;
-		gradientImpl(ff, refFit, point, order, eps, dfn, gradOut);
+		gradientImpl(ff, numThreads, refFit, point, order, eps, dfn, gradOut);
 		break;}
 	default: Rf_error("Unknown gradient algorithm %d", algo);
 	}
@@ -112,6 +119,7 @@ template <typename T1, typename T2, typename T3, typename T4, typename T5>
 void jacobianImpl(T1 ff,  Eigen::MatrixBase<T2> &ref, Eigen::MatrixBase<T3> &point,
 		  int numIter, const double eps, T4 dfn, Eigen::MatrixBase<T5> &jacobiOut)
 {
+	// TODO evaluate jacobian in parallel
 	for (int px=0; px < int(point.size()); ++px) {
 		double offset = std::max(fabs(point[px] * eps), eps);
 		Eigen::MatrixXd Gaprox(ref.size(), numIter);
