@@ -265,6 +265,9 @@ void omxData::newDataStatic(omxState *state, SEXP dataObject)
 	if(OMX_DEBUG) {mxLog("Processing Observed Thresholds Matrix.");}
 	{ScopedProtect p1(dataLoc, R_do_slot(dataObject, Rf_install("thresholds")));
 	od->obsThresholdsMat = omxNewMatrixFromRPrimitive(dataLoc, state, 0, 0);
+	if (!strEQ(od->obsThresholdsMat->getType(), "matrix")) {
+		Rf_error("Observed thresholds must be constant");
+	}
 	}
 
 	if(od->obsThresholdsMat->rows == 1 && od->obsThresholdsMat->cols == 1 && 
@@ -288,7 +291,6 @@ void omxData::newDataStatic(omxState *state, SEXP dataObject)
 
 		for(int i = 0; i < od->obsThresholdsMat->cols; i++) {
 			omxThresholdColumn tc;
-			tc.matrix = od->obsThresholdsMat;
 			tc.column = columns[i];
 			tc.numThresholds = levels[i];
 			od->thresholdCols.push_back(tc);
@@ -522,32 +524,10 @@ void omxContiguousDataRow(omxData *od, int row, int start, int len, omxMatrix* o
 	memcpy(dest, source, sizeof(double) * len);
 }
 
-void omxDataRow(omxData *od, int row, omxMatrix* colList, omxMatrix* om) {
-
-	if(colList == NULL || row >= od->rows) Rf_error("Invalid row or colList");
-
-	if(om == NULL) Rf_error("Must provide an output matrix");
-
-	int numcols = colList->cols;
-	if(od->dataMat != NULL) { // Matrix Object
-		omxMatrix* dataMat = od->dataMat;
-		for(int j = 0; j < numcols; j++) {
-			omxSetMatrixElement(om, 0, j, omxMatrixElement(dataMat, row, 
-								       omxVectorElement(colList, j)));
-		}
-	} else {		// Data Frame object
-		for(int j = 0; j < numcols; j++) {
-			int col = omxVectorElement(colList, j);
-			omxSetMatrixElement(om, 0, j, omxDoubleDataElement(od, row, col));
-		}
-	}
-}
-
-void omxDataRow(omxExpectation *ex, int row, omxMatrix* om)
+void omxDataRow(omxData *od, int row, omxMatrix* colList, omxMatrix* om)
 {
-	omxData *od = ex->data;
-	omxMatrix *colList = ex->dataColumns;
-	omxDataRow(od, row, colList, om);
+	EigenVectorAdaptor ecl(colList);
+	omxDataRow(od, row, ecl, om);
 }
 
 int omxDataIndex(omxData *od, int row) {
@@ -944,10 +924,8 @@ static void markDefVarDependencies(omxState* os, omxDefinitionVar* defVar)
 	}
 }
 
-bool omxData::handleDefinitionVarList(omxState *state, int row)
+bool omxData::loadDefVars(omxState *state, int row)
 {
-	if(OMX_DEBUG_ROWS(row)) { mxLog("Processing Definition Vars for row %d", row); }
-	
 	bool changed = false;
 	for (int k=0; k < int(defVars.size()); ++k) {
 		double newDefVar = omxDoubleDataElement(this, row, defVars[k].column);
@@ -956,6 +934,7 @@ bool omxData::handleDefinitionVarList(omxState *state, int row)
 		}
 		changed |= defVars[k].loadData(state, newDefVar);
 	}
+	if (changed && OMX_DEBUG_ROWS(row)) { mxLog("Processing Definition Vars for row %d", row); }
 	return changed;
 }
 
@@ -966,21 +945,29 @@ void omxData::loadFakeData(omxState *state, double fake)
 	}
 }
 
+bool omxData::CompareDefVarInMatrix(int lrow, int rrow, omxMatrix *mat, bool &mismatch)
+{
+	int mnum = ~mat->matrixNumber;
+	mismatch = true;
+	for (int dx=0; dx < int(defVars.size()); ++dx) {
+		omxDefinitionVar &dv = defVars[dx];
+		if (dv.matrix != mnum) continue;
+		double lval = omxDoubleDataElement(this, lrow, dv.column);
+		double rval = omxDoubleDataElement(this, rrow, dv.column);
+		if (lval != rval) return lval < rval;
+	}
+	mismatch = false;
+	return false;
+}
+
 bool omxDefinitionVar::loadData(omxState *state, double val)
 {
-	// We only need to check the first location because
-	// all locations will have the same value.
-	for(int l = 0; l < numLocations; l++) {
-		int matrixNumber = matrices[l];
-		int matrow = rows[l];
-		int matcol = cols[l];
-		omxMatrix *matrix = state->matrixList[matrixNumber];
-		if (val == omxMatrixElement(matrix, matrow, matcol)) return false;
-		omxSetMatrixElement(matrix, matrow, matcol, val);
-		if (OMX_DEBUG) {
-			mxLog("Load data %f into %s[%d,%d], state[%d]",
-			      val, matrix->name(), matrow, matcol, state->getId());
-		}
+	omxMatrix *mat = state->matrixList[matrix];
+	if (val == omxMatrixElement(mat, row, col)) return false;
+	omxSetMatrixElement(mat, row, col, val);
+	if (OMX_DEBUG) {
+		mxLog("Load data %f into %s[%d,%d], state[%d]",
+		      val, mat->name(), row, col, state->getId());
 	}
 	markDefVarDependencies(state, this);
 	return true;
