@@ -36,7 +36,6 @@
 
 #include "omxDefines.h"
 #include <Eigen/Core>
-#include "omxBLAS.h"
 
 struct populateLocation {
 	int from;
@@ -88,9 +87,6 @@ class omxMatrix {
 	unsigned cleanVersion;
 	unsigned version;
 
-	int originalRows; //deprecated
-	int originalCols; //deprecated
-
 /* For Algebra Functions */				// At most, one of these may be non-NULL.
 	omxAlgebra* algebra;				// If it's not an algebra, this is NULL.
 	omxFitFunction* fitFunction;		// If it's not a fit function, this is NULL.
@@ -111,6 +107,25 @@ class omxMatrix {
 		if (fitFunction) what = "fitfunction";
 		return what;
 	}
+	void copyAttr(omxMatrix *src);
+};
+
+void omxEnsureColumnMajor(omxMatrix *mat);
+
+inline double *omxMatrixDataColumnMajor(omxMatrix *mat)
+{
+	omxEnsureColumnMajor(mat);
+	return mat->data;
+}
+
+struct EigenMatrixAdaptor : Eigen::Map< Eigen::MatrixXd > {
+	EigenMatrixAdaptor(omxMatrix *mat) :
+	  Eigen::Map< Eigen::MatrixXd >(omxMatrixDataColumnMajor(mat), mat->rows, mat->cols) {}
+};
+
+struct EigenVectorAdaptor : Eigen::Map< Eigen::VectorXd > {
+	EigenVectorAdaptor(omxMatrix *mat) :
+	  Eigen::Map< Eigen::VectorXd >(mat->data, mat->rows * mat->cols) {}
 };
 
 // If you call these functions directly then you need to free the memory with omxFreeMatrix.
@@ -144,7 +159,6 @@ void omxResizeMatrix(omxMatrix *source, int nrows, int ncols);
 	omxMatrix* omxFillMatrixFromRPrimitive(omxMatrix* om, SEXP rObject, omxState *state,
 		unsigned short hasMatrixNumber, int matrixNumber); 								// Populate an omxMatrix from an R object
 	void omxTransposeMatrix(omxMatrix *mat);												// Transpose a matrix in place.
-void omxEnsureColumnMajor(omxMatrix *mat);
 	void omxToggleRowColumnMajor(omxMatrix *mat);										// Transform row-major into col-major and vice versa 
 
 /* Function wrappers that switch based on inclusion of algebras */
@@ -167,6 +181,8 @@ void setVectorError(int index, int numrow, int numcol);
 void matrixElementError(int row, int col, int numrow, int numcol);
 void vectorElementError(int index, int numrow, int numcol);
 
+bool omxNeedsUpdate(omxMatrix *matrix);
+
 OMXINLINE static bool omxMatrixIsDirty(omxMatrix *om) { return om->cleanVersion != om->version; }
 OMXINLINE static bool omxMatrixIsClean(omxMatrix *om) { return om->cleanVersion == om->version; }
 OMXINLINE static int omxGetMatrixVersion(omxMatrix *om) { return om->version; }
@@ -260,41 +276,84 @@ static OMXINLINE double omxUnsafeVectorElement(omxMatrix *om, int index) {
 	return om->data[index];
 }
 
+// In the process of debugging some other problem, I noticed that
+// DDEBUGMX_ALGEBRA causes models/passing/SimpleAlgebraCIs.R to fail.
+//
+// I used GIT to bisect the problem. It seems b1e8136b758c838 broke
+// DDEBUGMX_ALGEBRA. In this commit, I changed the Fortran argument
+// passing from character to integer at request of some CRAN maintainers.
+// However, the change also appears to have broken something subtle in
+// Fortran/C communication.
+//
+// I am really not sure how to identify the underlying issue.
+//
+// Acting on the belief that it's better to have correct code than fast
+// code, I decided to replace omxBLAS.f with equivalent Eigen code.
+// omxBLAS was (supposedly) fast because it omitted the code to
+// properly handle NaN (in some cases).
 
-static OMXINLINE void omxDGEMM(bool transposeA, bool transposeB,		// result <- alpha * A %*% B + beta * C
+static OMXINLINE void omxDGEMM(unsigned short int transposeA, unsigned short int transposeB,		// result <- alpha * A %*% B + beta * C
 				double alpha, omxMatrix* a, omxMatrix *b, double beta, omxMatrix* result) {
-	int nrow = (transposeA?a->cols:a->rows);
-	int nmid = (transposeA?a->rows:a->cols);
-	int ncol = (transposeB?b->rows:b->cols);
 
-	int transa = !(transposeA ^ bool(a->colMajor));
-	int transb = !(transposeB ^ bool(b->colMajor));
-	F77_CALL(omxunsafedgemm)(&transa, &transb,
-							&(nrow), &(ncol), &(nmid),
-							&alpha, a->data, &(a->leading), 
-							b->data, &(b->leading),
-				 &beta, result->data, &(result->leading));
+	EigenMatrixAdaptor eA(a);
+	EigenMatrixAdaptor eB(b);
+	EigenMatrixAdaptor eResult(result);
+	Eigen::MatrixXd ccopy;
 
-	if(!result->colMajor) omxToggleRowColumnMajor(result);
+	if (beta) {
+		ccopy = eResult * beta;
+	}
+	if (!transposeA && !transposeB) {
+		eResult.derived() = alpha * eA * eB;
+	} else if (transposeA && !transposeB) {
+		eResult.derived() = alpha * eA.transpose() * eB;
+	} else if (!transposeA && transposeB) {
+		eResult.derived() = alpha * eA * eB.transpose();
+	} else {
+		eResult.derived() = alpha * eA.transpose() * eB.transpose();
+	}
+	if (beta) {
+		eResult.derived() += ccopy;
+	}
+	result->colMajor = true;
+	omxMatrixLeadingLagging(result);
 }
 
-static OMXINLINE void omxDGEMV(bool transposeMat, double alpha, omxMatrix* mat,	// result <- alpha * A %*% B + beta * C
+static OMXINLINE void omxDGEMV(bool transpose, double alpha, omxMatrix* mat,	// result <- alpha * A %*% B + beta * C
 				omxMatrix* vec, double beta, omxMatrix*result) {							// where B is treated as a vector
-	int onei = 1;
-	int nrows = mat->rows;
-	int ncols = mat->cols;
-	if(OMX_DEBUG_MATRIX) {
-		int nVecEl = vec->rows * vec->cols;
-		// mxLog("DGEMV: %c, %d, %d, %f, 0x%x, %d, 0x%x, %d, 0x%x, %d\n", *(transposeMat?mat->minority:mat->majority), (nrows), (ncols), 
-        	// alpha, mat->data, (mat->leading), vec->data, onei, beta, result->data, onei); //:::DEBUG:::
-		if((transposeMat && nrows != nVecEl) || (!transposeMat && ncols != nVecEl)) {
-			Rf_error("Mismatch in vector/matrix multiply: %s (%d x %d) * (%d x 1).\n", (transposeMat?"transposed":""), mat->rows, mat->cols, nVecEl); // :::DEBUG:::
+	EigenMatrixAdaptor eMat(mat);
+	EigenVectorAdaptor eVec(vec);
+	EigenMatrixAdaptor eResult(result);
+	Eigen::VectorXd vcopy;
+
+	//mxLog("%dx%d = %dx%d * %d (t=%d)", eResult.rows(), eResult.cols(),
+	//eMat.rows(), eMat.cols(), eVec.rows(), transpose);
+
+	if (beta) {
+		vcopy = eResult * beta;
+	}
+	int rows = transpose? eMat.cols() : eMat.rows();
+	if (eResult.cols() == rows) {
+		if (transpose) {
+			eResult.derived() = (alpha * eMat.transpose() * eVec).transpose();
+		} else {
+			eResult.derived() = (alpha * eMat * eVec).transpose();
+		}
+		if (beta) {
+			eResult.derived() += vcopy.transpose();
+		}
+	} else {
+		if (transpose) {
+			eResult.derived() = alpha * eMat.transpose() * eVec;
+		} else {
+			eResult.derived() = alpha * eMat * eVec;
+		}
+		if (beta) {
+			eResult.derived() += vcopy;
 		}
 	}
-	int trans = !(transposeMat ^ bool(mat->colMajor));
-	F77_CALL(omxunsafedgemv)(&trans, &(nrows), &(ncols), 
-	        &alpha, mat->data, &(mat->leading), vec->data, &onei, &beta, result->data, &onei);
-	if(!result->colMajor) omxToggleRowColumnMajor(result);
+	result->colMajor = true;
+	omxMatrixLeadingLagging(result);
 }
 
 static OMXINLINE void omxDSYMV(double alpha, omxMatrix* mat,            // result <- alpha * A %*% B + beta * C
@@ -366,8 +425,6 @@ double omxMaxAbsDiff(omxMatrix *m1, omxMatrix *m2);
 
 void checkIncreasing(omxMatrix* om, int column, int count, FitContext *fc);
 
-void omxStandardizeCovMatrix(omxMatrix* cov, double* corList, double* weights, FitContext* fc);
-
 void omxMatrixHorizCat(omxMatrix** matList, int numArgs, omxMatrix* result);
 
 void omxMatrixVertCat(omxMatrix** matList, int numArgs, omxMatrix* result);
@@ -377,22 +434,6 @@ void omxMatrixTrace(omxMatrix** matList, int numArgs, omxMatrix* result);
 void expm_eigen(int n, double *rz, double *out);
 void logm_eigen(int n, double *rz, double *out);
 
-inline double *omxMatrixDataColumnMajor(omxMatrix *mat)
-{
-	omxEnsureColumnMajor(mat);
-	return mat->data;
-}
-
-struct EigenMatrixAdaptor : Eigen::Map< Eigen::MatrixXd > {
-	EigenMatrixAdaptor(omxMatrix *mat) :
-	  Eigen::Map< Eigen::MatrixXd >(omxMatrixDataColumnMajor(mat), mat->rows, mat->cols) {}
-};
-
-struct EigenVectorAdaptor : Eigen::Map< Eigen::VectorXd > {
-	EigenVectorAdaptor(omxMatrix *mat) :
-	  Eigen::Map< Eigen::VectorXd >(mat->data, mat->rows * mat->cols) {}
-};
-
 template <typename T>
 void mxPrintMat(const char *name, const Eigen::DenseBase<T> &mat)
 {
